#include-once

; #INDEX# =======================================================================================================================
; Title .........: _FileGetIcon
; AutoIt Version : 3.3.8.1
; Language ......: English
; Description ...: Function for getting file icons.
; Author(s) .....: Rob Saunders (therks)
; Dll(s) ........: shell32.dll, shlwapi.dll
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_FileGetIcon
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name...........: _FileGetIcon
; Description ...: Retrieve path to icon for a file type.
; Syntax.........: _FileGetIcon($sPath, $aPresetIcons = Default)
; Parameters ....: $sPath        - File/folder/drive path
;                  $aPresetIcons - A 10 element array to customize the preset icon returns:
;                     [0] is icon file path (default: @SystemDir & '\shell32.dll')
;                     [1] is ordinal for REMOVABLE drivetype (default: -8)
;                     [2] is ordinal for NETWORK drivetype (default: -10)
;                     [3] is ordinal for CDROM drivetype (default: -12)
;                     [4] is ordinal for RAMDISK drivetype (default: -13)
;                     [5] is ordinal for FIXED drivetype (default: -9)
;                     [6] is ordinal for UNC path (default: -86)
;                     [7] is ordinal for default EXE icon (default: -3)
;                     [8] is ordinal for default LNK icon (default: -30)
;                     [9] is ordinal for default unregistered icon (default: 1)
; Return values .: Success OR Failure: Returns 2 element array: [0] = Icon file path, [1] = icon index/ordinal.
;                  Failure: Sets @error:
;                  |1 - Path is drive root but cannot determine drive type
;                  |2 - Path is shortcut file but unable to read it
;                  |3 - The extension is not registered
;                  |4 - The filetype is not registered or has no DefaultIcon set
;                  |5 - Icon file does not exist and cannot be found
;                  |6 - Icon is invalid and cannot be loaded
; Author ........: Rob Saunders (rob at therks dot com)
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================

Func _FileGetIcon($sPath, $aPresetIcons = Default)
	Local $iExpandEnvStrings = Opt('ExpandEnvStrings', 1)
	Local $aOutput[2], $iError, $iExtended, $sDriveType, $sFileType, $sFileExt, $sIconPath
	Local Enum $ICO_DLL, $ICO_REMOVABLE, $ICO_NETWORK, $ICO_CDROM, $ICO_RAMDISK, $ICO_FIXED, $ICO_UNC, $ICO_EXE, $ICO_LNK, $ICO_GEN
	If UBound($aPresetIcons) <> 10 Then Dim $aPresetIcons[10] = [ @SystemDir & '\shell32.dll', -8, -10, -12, -13, -9, -86, -3, -30, 1 ]
	Local $aDllCall, $tIcon = DllStructCreate('int'), $pIcon = DllStructGetPtr($tIcon)

	; If path is a URL then filetype is the type of URL
	Local $aRegEx = StringRegExp($sPath, '^(https?|ftp|mailto)', 1)
	If @extended Then
		$sFileType = $aRegEx[0]
	Else
		If StringRegExp($sPath, '^[a-zA-Z]\:\\?$') Then
			; Path is the root of a drive.
			$iExtended = 1
			; Check drive type.
			$sDriveType = DriveGetType($sPath)
			If Not @error Then
				; Return a generic icon for the drive type.
				$aOutput[0] = $aPresetIcons[$ICO_DLL]
				If IsDeclared('ICO_' & $sDriveType) = -1 Then
					$aOutput[1] = $aPresetIcons[Eval('ICO_' & $sDriveType)]
				Else
					$aOutput[1] = $aPresetIcons[$ICO_FIXED]
				EndIf
			Else
				$iError = 1 ; Path is drive root but cannot determine drive type
			EndIf
		ElseIf StringRegExp($sPath, '^\\\\[^/\\:*?"<>|]+\\?$') Then
			; UNC root path. ie: \\ComputerName\
			$aOutput[0] = $aPresetIcons[$ICO_DLL]
			$aOutput[1] = $aPresetIcons[$ICO_UNC]
		ElseIf StringInStr(FileGetAttrib($sPath), 'D') Then
			; If path is a folder, set FileType to directory
			$sFileType = 'Directory'
			$iExtended = 2
		Else
			; Now we check the extension
			Local $iDot = StringInStr($sPath, '.', 0, -1) ; Find the last dot
			If $iDot Then
				$sFileExt = StringTrimLeft($sPath, $iDot) ; Get the extension
				If $sFileExt = 'lnk' Then
					$iExtended = 4
					; If path is a shortcut file, check it's parameters for a custom icon
					Local $aShortcut = FileGetShortcut($sPath)
					If @error Then
						$iError = 2 ; Path is shortcut file but unable to read it
					Else
						If Not $aShortcut[4] Then
							; If no custom icon set in shortcut file, rerun function on shortcut target path
							$aOutput = _FileGetIcon($aShortcut[0])
						Else
							; Return custom icon settings
							$aOutput[0] = $aShortcut[4]
							$aOutput[1] = -$aShortcut[5]
						EndIf
					EndIf
				Else
					; Get the filetype from the registry
					$sFileType = RegRead('HKCR\.' & $sFileExt, '')
					If @error Then $iError = 3 ; The extension is not registered
				EndIf
			EndIf
		EndIf
	EndIf

	If Not $iError And Not $aOutput[0] Then
		; We have no errors and no icon set yet
		$sIconPath = RegRead('HKCR\' & $sFileType & '\DefaultIcon', '')
		If @error Then
			$iError = 4 ; The filetype is not registered or has no DefaultIcon set
		Else
			; Some files refer to themselves for the icon
			$sIconPath = StringReplace($sIconPath, '%1', $sPath)
			; Split the icon filename from the index/ordinal
			$iExtended = 5
			Local $aSplit = StringRegExp($sIconPath, '^"?([^"]+)"?,(\-?[0-9]+)$', 1)
			If @error Then
				Local $aOutput[2] = [ FileGetLongName(StringReplace($sIconPath, '"', '')), 0 ]
			Else
				Local $aOutput[2] = [ FileGetLongName($aSplit[0]), -$aSplit[1] ]
				If $aOutput[1] < 0 Then $aOutput[1] -= 1
			EndIf
		EndIf
	EndIf

	; If the icon file doesn't exist then try to find it
	If $aOutput[0] And Not FileExists($aOutput[0]) Then
		$iError = 5 ; Icon file does not exist and cannot be found
		$aDllCall = DllCall('shlwapi.dll', 'bool', 'PathFindOnPathW', 'wstr', $aOutput[0], 'ptr', Ptr(0))
		If Not @error And $aDllCall[0] Then
			$aOutput[0] = $aDllCall[1]
			$iError = 0 ; We found it ;)
		EndIf
	EndIf

	If Not $iError Then
		; Finally we test the icon to see if it works
		$aDllCall = DllCall('shell32.dll', 'uint', 'ExtractIconExW', 'wstr', $aOutput[0], 'int', $aOutput[1], 'handle', $pIcon, 'handle', 0, 'uint', 1)
		If @error Or Not $aDllCall[0] Then $iError = 6 ; Icon is invalid and cannot be loaded
	EndIf

	; If any errors return a generic file icon
	If $iError Then
		If $sFileExt = 'exe' Or $sFileExt = 'scr' Then
			Local $aOutput[2] = [ $aPresetIcons[$ICO_DLL], $aPresetIcons[$ICO_EXE] ]
		ElseIf $sFileExt = 'lnk' Then
			Local $aOutput[2] = [ $aPresetIcons[$ICO_DLL], $aPresetIcons[$ICO_LNK] ]
		Else
			Local $aOutput[2] = [ $aPresetIcons[$ICO_DLL], $aPresetIcons[$ICO_GEN] ]
		EndIf
	EndIf

	Opt('ExpandEnvStrings', $iExpandEnvStrings)
	Return SetError($iError, $iExtended, $aOutput)
EndFunc
